"""
import all wrapper function
"""


# import mouse
from je_auto_control.wrapper.auto_control_mouse import click_mouse
from je_auto_control.wrapper.auto_control_mouse import mouse_table
from je_auto_control.wrapper.auto_control_mouse import position
from je_auto_control.wrapper.auto_control_mouse import press_mouse
from je_auto_control.wrapper.auto_control_mouse import release_mouse
from je_auto_control.wrapper.auto_control_mouse import scroll
from je_auto_control.wrapper.auto_control_mouse import set_position
from je_auto_control.wrapper.auto_control_mouse import special_table

# import keyboard
from je_auto_control.wrapper.auto_control_keyboard import keys_table
from je_auto_control.wrapper.auto_control_keyboard import press_key
from je_auto_control.wrapper.auto_control_keyboard import release_key
from je_auto_control.wrapper.auto_control_keyboard import type_key
from je_auto_control.wrapper.auto_control_keyboard import check_key_is_press
from je_auto_control.wrapper.auto_control_keyboard import write
from je_auto_control.wrapper.auto_control_keyboard import hotkey

# import screen
from je_auto_control.wrapper.auto_control_screen import size
from je_auto_control.wrapper.auto_control_screen import screenshot

# import image
from je_auto_control.wrapper.auto_control_image import locate_all_image
from je_auto_control.wrapper.auto_control_image import locate_image_center
from je_auto_control.wrapper.auto_control_image import locate_and_click

# Critical
from je_auto_control.utils.critical_exit.critcal_exit import CriticalExit

# Exception
from je_auto_control.utils.exception.exceptions import AutoControlException
from je_auto_control.utils.exception.exceptions import AutoControlKeyboardException
from je_auto_control.utils.exception.exceptions import AutoControlMouseException
from je_auto_control.utils.exception.exceptions import AutoControlCantFindKeyException
from je_auto_control.utils.exception.exceptions import AutoControlScreenException
from je_auto_control.utils.exception.exceptions import ImageNotFoundException
from je_auto_control.utils.exception.exceptions import AutoControlJsonActionException
from je_auto_control.utils.exception.exceptions import AutoControlRecordException
from je_auto_control.utils.exception.exceptions import AutoControlActionNullException
from je_auto_control.utils.exception.exceptions import AutoControlActionException

# test_record
from je_auto_control.wrapper.auto_control_record import record
from je_auto_control.wrapper.auto_control_record import stop_record

# json
from je_auto_control.utils.json.json_file import read_action_json
from je_auto_control.utils.json.json_file import write_action_json

# executor
from je_auto_control.utils.executor.action_executor import execute_action
from je_auto_control.utils.executor.action_executor import execute_files
from je_auto_control.utils.executor.action_executor import executor
from je_auto_control.utils.executor.action_executor import add_command_to_executor

# timeout
from je_auto_control.utils.timeout.multiprocess_timeout import multiprocess_timeout
# test record
from je_auto_control.utils.test_record.record_test_class import test_record_instance

# utils image
from je_auto_control.wrapper.auto_control_image import screenshot

# html report
from je_auto_control.utils.html_report.html_report_generate import generate_html

# file process
from je_auto_control.utils.file_process.get_dir_file_list import get_dir_files_as_list
from je_auto_control.utils.file_process.create_project_structure import create_template_dir
# socket server
from je_auto_control.utils.socket_server.auto_control_socket_server import start_autocontrol_socket_server



import sys
from typing import Tuple

from je_auto_control.utils.exception.exceptions import AutoControlException
from je_auto_control.utils.exception.exception_tag import linux_import_error

if sys.platform not in ["linux", "linux2"]:
    raise AutoControlException(linux_import_error)

from je_auto_control.linux_with_x11.core.utils.x11_linux_display import display


def size() -> Tuple[int, int]:
    """
    get screen size
    """
    return display.screen().width_in_pixels, display.screen().height_in_pixels



# error tags
je_auto_control_error: str = "Auto control error"
je_auto_control_critical_exit_error: str = "Auto control critical exit error"
# os tags
linux_import_error: str = "should be only loaded on linux"
osx_import_error: str = "should be only loaded on MacOS"
windows_import_error: str = "should be only loaded on windows"
macos_record_error: str = "macos cant use recorder"
# keyboard tags
keyboard_error: str = "Auto control keyboard error"
keyboard_press_key: str = "keyboard press key error"
keyboard_release_key: str = "keyboard release key error"
keyboard_type_key: str = "keyboard type key error"
keyboard_write: str = "keyboard write error"
keyboard_write_cant_find: str = "keyboard write error cant find key"
keyboard_hotkey: str = "keyboard hotkey error"
# mouse tags
mouse_error: str = "Auto control mouse error"
mouse_get_position: str = "mouse get position error"
mouse_set_position: str = "mouse set position error"
mouse_press_mouse: str = "mouse press mouse error"
mouse_release_mouse: str = "mouse release key error"
mouse_click_mouse: str = "mouse click mouse error"
mouse_scroll: str = "mouse scroll error"
mouse_wrong_value: str = "mouse value error"
# screen tags
screen_error: str = "Auto control screen error"
screen_get_size: str = "screen get size error"
screen_screenshot: str = "screen screenshot error"
# table tags
table_cant_find_key: str = "cant find key error"
# image tags
cant_find_image: str = "cant find image"
find_image_error_variable: str = "variable error"
# listener tags
listener_error: str = "Auto control listener error"
# test_record tags
record_queue_error: str = "cant get test_record queue its none are you using stop test_record before test_record"
record_not_found_action_error: str = "test_record action not found"
# json action file tag
cant_execute_action_error: str = "cant execute action"
cant_find_json_error: str = "cant find json file"
cant_save_json_error: str = "cant save json file"
action_is_null_error: str = "json action is null"
# timeout tag
timeout_need_on_main_error: str = "should put timeout function on main"
# HTML
html_generate_no_data_tag: str = "record is None"
# add command
add_command_exception_tag: str = "command value type should be as method or function"
# executor
executor_list_error: str = "executor receive wrong data list is none or wrong type"
# argparse
argparse_get_wrong_data: str = "argparse receive wrong data"



import sys

from je_auto_control.utils.exception.exception_tag import windows_import_error
from je_auto_control.utils.exception.exceptions import AutoControlException

if sys.platform not in ["win32", "cygwin", "msys"]:
    raise AutoControlException(windows_import_error)

from ctypes import *
from ctypes.wintypes import MSG

from threading import Thread

from queue import Queue

_user32: windll.user32 = windll.user32
_kernel32: windll.kernel32 = windll.kernel32
_wm_keydown: int = 0x100


class Win32KeyboardListener(Thread):

    def __init__(self):
        super().__init__()
        self.setDaemon(True)
        self.hooked: [None, int] = None
        self.record_queue: [None, Queue] = None
        self.record_flag: bool = False
        self.hook_event_code_int: int = 13

    def _set_win32_hook(self, point) -> bool:
        self.hooked = _user32.SetWindowsHookExA(
            self.hook_event_code_int,
            point,
            0,
            0
        )
        if not self.hooked:
            return False
        return True

    def _remove_win32_hook_proc(self) -> None:
        if self.hooked is None:
            return
        _user32.UnhookWindowsHookEx(self.hooked)
        self.hooked = None

    def _win32_hook_proc(self, code, w_param, l_param) -> _user32.CallNextHookEx:
        if w_param is not _wm_keydown:
            return _user32.CallNextHookEx(self.hooked, code, w_param, l_param)
        if self.record_flag is True:
            # int to hex
            temp = hex(l_param[0] & 0xFFFFFFFF)
            self.record_queue.put(("type_key", int(temp, 16)))
        return _user32.CallNextHookEx(self.hooked, code, w_param, l_param)

    def _get_function_pointer(self, function) -> WINFUNCTYPE:
        win_function = WINFUNCTYPE(c_int, c_int, c_int, POINTER(c_void_p))
        return win_function(function)

    def _start_listener(self) -> None:
        pointer = self._get_function_pointer(self._win32_hook_proc)
        self._set_win32_hook(pointer)
        message = MSG()
        _user32.GetMessageA(byref(message), 0, 0, 0)

    def record(self, want_to_record_queue) -> None:
        self.record_flag = True
        self.record_queue = want_to_record_queue
        self.start()

    def stop_record(self) -> Queue:
        self.record_flag = False
        self._remove_win32_hook_proc()
        return self.record_queue

    def run(self) -> None:
        self._start_listener()




# argparse
import argparse
import json
import sys

from je_auto_control.utils.exception.exception_tag import argparse_get_wrong_data
from je_auto_control.utils.exception.exceptions import AutoControlArgparseException
from je_auto_control.utils.json.json_file import read_action_json
from je_auto_control.utils.file_process.get_dir_file_list import get_dir_files_as_list
from je_auto_control.utils.executor.action_executor import execute_action
from je_auto_control.utils.executor.action_executor import execute_files

if __name__ == "__main__":
    try:
        def preprocess_execute_action(file_path: str):
            execute_action(read_action_json(file_path))


        def preprocess_execute_files(file_path: str):
            execute_files(get_dir_files_as_list(file_path))

        def preprocess_read_str_execute_action(execute_str: str):
            execute_str = json.loads(execute_str)
            execute_action(execute_str)

        argparse_event_dict = {
            "execute_file": preprocess_execute_action,
            "execute_dir": preprocess_execute_files,
            "execute_str": preprocess_read_str_execute_action
        }
        parser = argparse.ArgumentParser()
        parser.add_argument("-e", "--execute_file", type=str, help="choose action file to execute")
        parser.add_argument("-d", "--execute_dir", type=str, help="choose dir include action file to execute")
        parser.add_argument("--execute_str", type=str, help="execute json str")
        args = parser.parse_args()
        args = vars(args)
        for key, value in args.items():
            if value is not None:
                argparse_event_dict.get(key)(value)
        if all(value is None for value in args.values()):
            raise AutoControlArgparseException(argparse_get_wrong_data)
    except Exception as error:
        print(repr(error), file=sys.stderr)
        sys.exit(1)




import sys
from queue import Queue

from je_auto_control.utils.exception.exception_tag import osx_import_error
from je_auto_control.utils.exception.exceptions import AutoControlException

if sys.platform not in ["darwin"]:
    raise AutoControlException(osx_import_error)

from je_auto_control.osx.listener.osx_listener import osx_record
from je_auto_control.osx.listener.osx_listener import osx_stop_record

from je_auto_control.utils.exception.exceptions import AutoControlJsonActionException


class OSXRecorder(object):

    def record(self) -> None:
        osx_record()

    def stop_record(self) -> Queue:
        record_queue = osx_stop_record()
        if record_queue is None:
            raise AutoControlJsonActionException
        return osx_stop_record()


osx_recorder = OSXRecorder()



import sys

from je_auto_control.utils.exception.exception_tag import windows_import_error
from je_auto_control.utils.exception.exceptions import AutoControlException

if sys.platform not in ["win32", "cygwin", "msys"]:
    raise AutoControlException(windows_import_error)

import ctypes


def check_key_is_press(keycode: [int, str]) -> bool:
    if type(keycode) is int:
        temp: int = ctypes.windll.user32.GetKeyState(keycode)
    else:
        temp = ctypes.windll.user32.GetKeyState(ord(keycode))
    if temp > 1:
        return True
    return False



import sys
from typing import Tuple

from je_auto_control.utils.exception.exception_tag import keyboard_hotkey
from je_auto_control.utils.exception.exception_tag import keyboard_press_key
from je_auto_control.utils.exception.exception_tag import keyboard_release_key
from je_auto_control.utils.exception.exception_tag import keyboard_type_key
from je_auto_control.utils.exception.exception_tag import keyboard_write
from je_auto_control.utils.exception.exception_tag import keyboard_write_cant_find
from je_auto_control.utils.exception.exception_tag import table_cant_find_key
from je_auto_control.utils.exception.exceptions import AutoControlCantFindKeyException
from je_auto_control.utils.exception.exceptions import AutoControlKeyboardException
from je_auto_control.wrapper.platform_wrapper import keyboard
from je_auto_control.wrapper.platform_wrapper import keyboard_check
from je_auto_control.wrapper.platform_wrapper import keys_table
from je_auto_control.utils.test_record.record_test_class import record_action_to_list


def press_key(keycode: [int, str], is_shift: bool = False, skip_record: bool = False) -> str:
    """
    use to press a key still press to use release key
    or use critical exit
    return keycode
    :param keycode which keycode we want to press
    :param is_shift press shift True or False
    :param skip_record skip record on record total list True or False
    """
    param = locals()
    try:
        if type(keycode) is not int:
            try:
                keycode = keys_table.get(keycode)
            except AutoControlCantFindKeyException:
                raise AutoControlCantFindKeyException(table_cant_find_key)
        try:
            if sys.platform in ["win32", "cygwin", "msys", "linux", "linux2"]:
                keyboard.press_key(keycode)
            elif sys.platform in ["darwin"]:
                keyboard.press_key(keycode, is_shift=is_shift)
            if skip_record is False:
                record_action_to_list("press_key", param)
            return str(keycode)
        except AutoControlKeyboardException as error:
            if skip_record is False:
                record_action_to_list("press_key", param, repr(error))
            raise AutoControlKeyboardException(keyboard_press_key + " " + repr(error))
        except TypeError as error:
            if skip_record is False:
                record_action_to_list("press_key", param, repr(error))
            raise AutoControlKeyboardException(repr(error))
    except Exception as error:
        if skip_record is False:
            record_action_to_list("press_key", param, repr(error))
        else:
            raise AutoControlKeyboardException(repr(error))
        print(repr(error), file=sys.stderr)


def release_key(keycode: [int, str], is_shift: bool = False, skip_record: bool = False) -> str:
    """
    use to release pressed key return keycode
    :param keycode which keycode we want to release
    :param is_shift press shift True or False
    :param skip_record skip record on record total list True or False
    """
    param = locals()
    try:
        if type(keycode) is not int:
            try:
                keycode = keys_table.get(keycode)
            except AutoControlCantFindKeyException:
                raise AutoControlCantFindKeyException(table_cant_find_key)
        try:
            if sys.platform in ["win32", "cygwin", "msys", "linux", "linux2"]:
                keyboard.release_key(keycode)
            elif sys.platform in ["darwin"]:
                keyboard.release_key(keycode, is_shift=is_shift)
            if skip_record is False:
                record_action_to_list("release_key", param)
            return str(keycode)
        except AutoControlKeyboardException as error:
            if skip_record is False:
                record_action_to_list("release_key", param, repr(error))
            raise AutoControlKeyboardException(keyboard_release_key + " " + repr(error))
        except TypeError as error:
            if skip_record is False:
                record_action_to_list("release_key", param, repr(error))
            raise AutoControlKeyboardException(repr(error))
    except Exception as error:
        if skip_record is False:
            record_action_to_list("release_key", param, repr(error))
        else:
            raise AutoControlKeyboardException(repr(error))
        print(repr(error), file=sys.stderr)


def type_key(keycode: [int, str], is_shift: bool = False, skip_record: bool = False) -> str:
    """
    press and release key return keycode
    :param keycode which keycode we want to type
    :param is_shift press shift True or False
    :param skip_record skip record on record total list True or False
    """
    param = locals()
    try:
        try:
            press_key(keycode, is_shift, skip_record=True)
            release_key(keycode, is_shift, skip_record=True)
            if skip_record is False:
                record_action_to_list("type_key", param)
            return str(keycode)
        except AutoControlKeyboardException as error:
            if skip_record is False:
                record_action_to_list("type_key", param, repr(error))
            raise AutoControlKeyboardException(keyboard_type_key + " " + repr(error))
        except TypeError as error:
            if skip_record is False:
                record_action_to_list("type_key", param, repr(error))
            raise AutoControlKeyboardException(repr(error))
    except Exception as error:
        if skip_record is False:
            record_action_to_list("type_key", param, repr(error))
        print(repr(error), file=sys.stderr)


def check_key_is_press(keycode: [int, str]) -> bool:
    """
    use to check key is press return True or False
    :param keycode check key is press or not
    """
    param = locals()
    try:
        if type(keycode) is int:
            get_key_code = keycode
        else:
            get_key_code = keys_table.get(keycode)
        record_action_to_list("check_key_is_press", param)
        return keyboard_check.check_key_is_press(keycode=get_key_code)
    except Exception as error:
        record_action_to_list("check_key_is_press", param, repr(error))
        print(repr(error), file=sys.stderr)


def write(write_string: str, is_shift: bool = False) -> str:
    """
    use to press and release whole we get this function str
    return all press and release str
    :param write_string while string not on write_string+1 type_key(string)
    :param is_shift press shift True or False
    """
    param = locals()
    try:
        try:
            record_write_string = ""
            for single_string in write_string:
                try:
                    if keys_table.get(single_string) is not None:
                        record_write_string = "".join(
                            [
                                record_write_string,
                                type_key(single_string, is_shift, skip_record=True)
                            ]
                        )
                    else:
                        raise AutoControlKeyboardException(keyboard_write_cant_find)
                except AutoControlKeyboardException as error:
                    print(keyboard_write_cant_find, single_string, sep="\t", file=sys.stderr)
                    raise AutoControlKeyboardException(keyboard_write_cant_find)
            record_action_to_list("write", param)
            return record_write_string
        except AutoControlKeyboardException as error:
            raise AutoControlKeyboardException(keyboard_write + " " + repr(error))
    except Exception as error:
        record_action_to_list("write", param, repr(error))
        print(repr(error), file=sys.stderr)


def hotkey(key_code_list: list, is_shift: bool = False) -> Tuple[str, str]:
    """
    use to press and release all key on key_code_list
    then reverse list press and release again
    return [press_str_list, release_str_list]
    :param key_code_list press and release all key on list and reverse
    :param is_shift press shift True or False
    """
    param = locals()
    try:
        try:
            record_hotkey_press_string = ""
            record_hotkey_release_string = ""
            for key in key_code_list:
                record_hotkey_press_string = ",".join(
                    [
                        record_hotkey_press_string,
                        press_key(key, is_shift, skip_record=True)
                    ]
                )
            key_code_list.reverse()
            for key in key_code_list:
                record_hotkey_release_string = ",".join(
                    [
                        record_hotkey_release_string,
                        release_key(key, is_shift, skip_record=True)
                    ]
                )
            record_action_to_list("hotkey", param)
            return record_hotkey_press_string, record_hotkey_release_string
        except AutoControlKeyboardException as error:
            raise AutoControlKeyboardException(keyboard_hotkey + " " + repr(error))
    except Exception as error:
        record_action_to_list("hotkey", param, repr(error))
        print(repr(error), file=sys.stderr)



import sys
from typing import List, Union

from je_auto_control.utils.exception.exception_tag import windows_import_error
from je_auto_control.utils.exception.exceptions import AutoControlException

if sys.platform not in ["win32", "cygwin", "msys"]:
    raise AutoControlException(windows_import_error)

import ctypes

_user32: ctypes.windll.user32 = ctypes.windll.user32
_user32.SetProcessDPIAware()


def size() -> List[Union[int, int]]:
    """
    get screen size
    """
    return [_user32.GetSystemMetrics(0), _user32.GetSystemMetrics(1)]



import sys

from je_auto_control.utils.exception.exceptions import AutoControlException
from je_auto_control.utils.exception.exception_tag import linux_import_error

if sys.platform not in ["linux", "linux2"]:
    raise AutoControlException(linux_import_error)

from Xlib import X

from je_auto_control.linux_with_x11.listener.x11_linux_listener import x11_linux_record
from je_auto_control.linux_with_x11.listener.x11_linux_listener import x11_linux_stop_record

from queue import Queue

type_dict = {5: "mouse", 3: "type_key"}
detail_dict = {1: "mouse_left", 2: "mouse_middle", 3: "mouse_right"}


class X11LinuxRecorder(object):
    """
    test_record controller
    """
    def __init__(self):
        self.record_queue = None
        self.result_queue = None

    def record(self) -> None:
        """
        create a new queue and start test_record
        """
        self.record_queue = Queue()
        x11_linux_record(self.record_queue)

    def stop_record(self) -> None:
        """
        stop test_record
        make a format action queue
        """
        self.result_queue = x11_linux_stop_record()
        action_queue = Queue()
        for details in self.result_queue.queue:
            if details[0] == 5:
                action_queue.put((detail_dict.get(details[1]), details[2], details[3]))
            elif details[0] == 3:
                action_queue.put((type_dict.get(details[0]), details[1]))
        return action_queue


x11_linux_recoder = X11LinuxRecorder()


